/*
    ShutDown Filter for VirtualDub -- Shut down computer after finished processing
	Copyright (C) 2002 Karel Suhajda

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	The author can be contacted at:
	Karel Suhajda
	kasuha@post.cz
*/

#define _WIN32_WINNT 0x0500

#include <windows.h>
#include <stdio.h>
#include <commctrl.h>

#include "ScriptInterpreter.h"
#include "ScriptError.h"
#include "ScriptValue.h"

#include "resource.h"
#include "filter.h"

///////////////////////////////////////////////////////////////////////////

BOOL CALLBACK ConfigDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);

int  InitProc(FilterActivation *fa, const FilterFunctions *ff);
long ParamProc(FilterActivation *fa, const FilterFunctions *ff);
int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd);
int RunProc(const FilterActivation *fa, const FilterFunctions *ff);
int EndProc(FilterActivation *fa, const FilterFunctions *ff);
int StartProc(FilterActivation *fa, const FilterFunctions *ff);
void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str);
void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc);
bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen);

///////////////////////////////////////////////////////////////////////////

typedef struct MyFilterData {
	bool forced;
	int timeout;
} MyFilterData;

ScriptFunctionDef func_defs[]={
	{ (ScriptFunctionPtr)ScriptConfig, "Config", "0ii" },
    { NULL },
};

CScriptObject script_obj={
    NULL, func_defs
};

struct FilterDefinition filterDef_tutorial = {

    NULL, NULL, NULL,       // next, prev, module
    "shutdown",             // name
    "Shuts the PC down after the processing has finished.",
                            // desc
    "Karel Suhajda",               // maker
    NULL,                   // private_data
    sizeof(MyFilterData),   // inst_data_size

    InitProc,                   // initProc
    NULL,                   // deinitProc
    RunProc,        // runProc
    ParamProc,      // paramProc
    ConfigProc,     // configProc
    StringProc,     // stringProc
    NULL,      // startProc
    EndProc,        // endProc

    &script_obj,          // script_obj
    FssProc,        // fssProc

};

///////////////////////////////////////////////////////////////////////////

extern "C" int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat);
extern "C" void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff);

static FilterDefinition *fd_tutorial;

///////////////////////////////////////////////////////////////////////////

int __declspec(dllexport) __cdecl VirtualdubFilterModuleInit2(FilterModule *fm, const FilterFunctions *ff, int& vdfd_ver, int& vdfd_compat) {
    if (!(fd_tutorial = ff->addFilter(fm, &filterDef_tutorial, sizeof(FilterDefinition))))
        return 1;

	vdfd_ver = VIRTUALDUB_FILTERDEF_VERSION;
	vdfd_compat = VIRTUALDUB_FILTERDEF_COMPATIBLE;

    return 0;
}

void __declspec(dllexport) __cdecl VirtualdubFilterModuleDeinit(FilterModule *fm, const FilterFunctions *ff) {
    ff->removeFilter(fd_tutorial);
}

///////////////////////////////////////////////////////////////////////////

void ScriptConfig(IScriptInterpreter *isi, void *lpVoid, CScriptValue *argv, int argc) {
    FilterActivation *fa = (FilterActivation *)lpVoid;
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->forced=!!argv[0].asInt();
	mfd->timeout=argv[1].asInt();
}

bool FssProc(FilterActivation *fa, const FilterFunctions *ff, char *buf, int buflen) {
	MyFilterData *mfd=(MyFilterData *)fa->filter_data;

	return(0<_snprintf(buf,buflen,"Config(%d,%d)",mfd->forced,mfd->timeout));
}

void StringProc(const FilterActivation *fa, const FilterFunctions *ff, char *str) {
    MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	sprintf(str, " (%s, %d s)",mfd->forced?"forced":"graceful",mfd->timeout);
}

int InitProc(FilterActivation *fa, const FilterFunctions *ff) {
	MyFilterData *mfd = (MyFilterData *)fa->filter_data;

	mfd->forced = false;
	mfd->timeout = 60;
	return 0;
}


int ConfigProc(FilterActivation *fa, const FilterFunctions *ff, HWND hwnd) {
    return DialogBoxParam(fa->filter->module->hInstModule, MAKEINTRESOURCE(IDD_CONFIG), hwnd, ConfigDlgProc, (LPARAM)fa->filter_data);
}

long ParamProc(FilterActivation *fa, const FilterFunctions *ff) {
	fa->dst.offset  = fa->src.offset;
	fa->dst.modulo  = fa->src.modulo;
	fa->dst.pitch   = fa->src.pitch;

	return 0;
}

int RunProc(const FilterActivation *fa, const FilterFunctions *ff){
	return 0;
}

void RefreshSlider(HWND hdlg) {
	char string[20];
	sprintf(string,"%d",SendMessage(GetDlgItem(hdlg,IDC_TIMEOUT),TBM_GETPOS,0,0));
	SetDlgItemText(hdlg,IDC_TEXT,string);
}

BOOL CALLBACK ConfigDlgProc (HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd;
	int i;

	switch(msg) {
    case WM_INITDIALOG:
        SetWindowLong(hdlg, DWL_USER, lParam);
        mfd = (MyFilterData *)lParam;
		
		if (mfd->forced) CheckDlgButton(hdlg,IDC_FORCED,BST_CHECKED);
		if (!mfd->forced) CheckDlgButton(hdlg,IDC_GRACEFUL,BST_CHECKED);

		SendDlgItemMessage(hdlg,IDC_TIMEOUT,TBM_SETRANGE,(WPARAM)TRUE,MAKELONG(2,120));
		SendDlgItemMessage(hdlg,IDC_TIMEOUT,TBM_SETPOS,(WPARAM)TRUE,mfd->timeout);
		for (i=10;i<120;i+=10) SendDlgItemMessage(hdlg,IDC_TIMEOUT,TBM_SETTIC, 0, i);

		RefreshSlider(hdlg);
        return TRUE;
    case WM_COMMAND:
		switch (HIWORD(wParam)) {
		case BN_CLICKED:
			switch(LOWORD(wParam)) {
			case IDOK:
			    mfd = (MyFilterData *)GetWindowLong(hdlg, DWL_USER);

				mfd->timeout = SendMessage(GetDlgItem(hdlg, IDC_TIMEOUT), TBM_GETPOS,0,0);
				mfd->forced = !!IsDlgButtonChecked(hdlg,IDC_FORCED);
				EndDialog(hdlg, 0);
				return TRUE;
			case IDCANCEL:
	            EndDialog(hdlg, 1);
		        return TRUE;
			}
		}
		break;
	case WM_NOTIFY:
		switch (((LPNMHDR)lParam)->code) {
		case NM_CUSTOMDRAW:
			if (((LPNMCUSTOMDRAW)lParam)->dwDrawStage==CDDS_PREPAINT) {
				switch (((NMHDR *)lParam)->idFrom) {
				case IDC_TIMEOUT:
					RefreshSlider(hdlg);
					return TRUE;
				}
			}
			break;
		}
		break;
	}
	return FALSE;
}

BOOL CALLBACK DialogProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	MyFilterData *mfd;

	switch (msg) {
	case WM_INITDIALOG:
        mfd = (MyFilterData *)lParam;
		SetTimer(hdlg, 1, 250, NULL);
		SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_SETRANGE, 0, MAKELPARAM(0, mfd->timeout*4));
		SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_SETPOS, 0, 0);
		SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_SETSTEP, 1, 0);
		return TRUE;

	case WM_TIMER:
		SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_STEPIT, 0, 0);
		if (SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_GETRANGE, FALSE, NULL) <= SendDlgItemMessage(hdlg, IDC_PROGRESS, PBM_GETPOS, 0,0)) {
			KillTimer(hdlg, 1);
			EndDialog(hdlg, IDOK);
		}
		return TRUE;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
		/* OK key removed
		case IDOK:
			EndDialog(hdlg, IDOK);
			KillTimer(hdlg, 1);
			return TRUE;
		*/
		case IDCANCEL:
			KillTimer(hdlg, 1);
			EndDialog(hdlg, IDCANCEL);
			return TRUE;
		}
	}
	return FALSE;
}

int EndProc(FilterActivation *fa, const FilterFunctions *ff){

	HANDLE hToken;              // handle to process token 
	TOKEN_PRIVILEGES tkp;       // pointer to token structure 
    bool ok; 
//	BOOL fResult;               // system shutdown flag 
 
	// Get the current process token handle so we can get shutdown 
	// privilege. 
 
	ok = !!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &hToken);
	
	if (ok) {
		// Get the LUID for shutdown privilege. 
		LookupPrivilegeValue(NULL, SE_SHUTDOWN_NAME, &tkp.Privileges[0].Luid); 
 
		tkp.PrivilegeCount = 1;  // one privilege to set    
		tkp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED; 
 
		// Get shutdown privilege for this process. 
 
		AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, (PTOKEN_PRIVILEGES) NULL, 0); 
 
		// Cannot test the return value of AdjustTokenPrivileges. 
 
		ok = (GetLastError() == ERROR_SUCCESS); 
	}

	if (IDOK == DialogBoxParam(fa->filter->module->hInstModule,
				               MAKEINTRESOURCE(IDD_SHUTDOWN), NULL,
						       DialogProc, (LPARAM)fa->filter_data)) {
		ExitWindowsEx(EWX_POWEROFF | EWX_SHUTDOWN | (((MyFilterData *)fa->filter_data)->forced?EWX_FORCE:EWX_FORCEIFHUNG),0);
	} 

	// Disable shutdown privilege. 
	
	if (ok) {
		tkp.Privileges[0].Attributes = 0; 
		AdjustTokenPrivileges(hToken, FALSE, &tkp, 0, 
			(PTOKEN_PRIVILEGES) NULL, 0); 
	}

	return 0;
}
